--[[
-- Copyright 2017 AKA Russell
--
-- This file is part of Scarlet.
--
-- Scarlet is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- Scarlet is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with Scarlet.  If not, see <http://www.gnu.org/licenses/>.
--]]
--[[
-- This module defines the characters in the game as Lua tables.
--
-- This module must include each character as a top level entity mapping system
-- name to the character's definition.
--]]

local M = {}

local ai = require 'combat/ai'
local anklyanapowers = require 'data/anklyanapowers'
local attack = require 'combat/attack'

local brightpowers = require 'data/brightpowers'
local buzzsawpowers = require 'data/buzzsawpowers'

local characters = require 'characters/characters'
local combat = require 'combat/combat'
local combatstateops = require 'combat/combatstate'

local items = require 'data/items'

local leopardpowers = require 'data/leopardpowers'
local log = require 'log'

local poseidonpowers = require 'data/poseidonpowers'
local princessai = require 'data/princessai'
local prometheuspowers = require 'data/prometheuspowers'

local power = require 'combat/powers'
local princesspowers = require 'data/princesspowers'

local scarletai = require 'data/scarletai'
local scarletchoices = require 'data/scarletchoices'
local scarletfunctions = require 'scarletfunctions'
local scarletmoonpowers = require 'data/scarletmoonpowers'
local scarlettables = require 'scarlettables'
local statuses = require 'data/statuses'
local statusops = require 'combat/statuses'
local succubuspowers = require 'data/succubuspowers'

local target = require 'combat/target'
local tempestaspowers = require './data/tempestaspowers'

local statistickeys = {"strength", "speed", "power", "energy", "maxenergy"}
local slotkeys = {"upper", "lower", "underwear"}

local function statistics(statstable)
    statstable._keys = statistickeys
    return statstable
end

local function clothing(clothingtable)
    clothingtable._keys = slotkeys
    return clothingtable
end

local blackdress = assert(items["black dress"])

local pcstatistickeys = {}
for i, key in ipairs(statistickeys) do
    pcstatistickeys[i] = key
end
table.insert(pcstatistickeys, "spanker")
table.insert(pcstatistickeys, "spankee")
table.insert(pcstatistickeys, "victory")
table.insert(pcstatistickeys, "defeat")

M.pc = characters.character {
    name="pc",
    printedname="Scarlette",
    statistics={
        strength=2,
        speed=2,
        power=2,
        maxenergy=30,
        energy=30,
        spanker=0,
        spankee=0,
        victory=0,
        defeat=0,
        _keys=pcstatistickeys,
    },
    powers={_keys={}},
    pc=true,
    choice=combat.basechoice(),
    clothing={
        upper=blackdress,
        lower=blackdress,
        underwear=assert(items.thong),
        _keys=slotkeys
    },
    attributes={
        gender="female",
        hairlength="backlength",
        hairstyle="french braid",
        haircolor="brown",
        skincolor="peach",
        height="average",
        musculature="soft",
        bodytype="voluptuous"
    }
}

M.buzzsaw = characters.character {
    name="buzzsaw",
    printedname="Buzzsaw",
    combatname="Buzzsaw",
    statistics={
        strength=2,
        speed=2,
        power=1,
        maxenergy=25,
        energy=25,
        _keys=statistickeys
    },
    powers={
        buzzsaw=buzzsawpowers.buzzsaw,
        chainwhip=buzzsawpowers.chainwhip,
        _keys={"buzzsaw", "chainwhip", }
    },
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.buzzsaw,
    attributes={
        gender="female",
        height="tall",
    },
    clothing=clothing {
        upper=items.studdedleatherjacket,
        lowere=items.blackjeanswithchain,
        underwear=items.studdedthong
    }
}

M.whistlerfemale = characters.character {
    name="whistlerfemale",
    printedname="Whistler",
    combatname="Whistler(F)",
    concisename="W(F)",
    choice=combat.basechoice(),
    attributes={gender="female"},
    statistics={
        strength=1,
        speed=1,
        power=0,
        maxenergy=1,
        energy=1,
        _keys=statistickeys
    },
    clothing=clothing {
        upper=items.vneckedtshirt,
        lower=items.baggyjeans,
        underwear=items.thong
    }
}

M.whistlermale = scarlettables.copy(M.whistlerfemale)
M.whistlermale.name = "whistlermale"
M.whistlermale.printedname = "Whistler"
M.whistlermale.combatname = "Whistler(M)"
M.whistlermale.concisename = "W(M)"
M.whistlermale.attributes.gender = "male"
M.whistlermale.clothing.underwear = items.boxers

M.whistlergunnerfemale= characters.character {
    name="whistlergunnerfemale",
    printedname="Whistler Gunner",
    combatname="Whistler Gunner(F)",
    concisename="WG(F)",
    attributes={gender="female"},
    choice=combat.basechoice(),
    choicecounts={
        shoot={count=1, page=scarletchoices.shoot},
        power={count=0, page=power.power},
        _keys={"shoot", "power"}
    },
    statistics=statistics {
        strength=1,
        speed=1,
        power=1,
        maxenergy=1,
        energy=1,
    },
    clothing=clothing {
        upper=items.tshirt,
        lower=items.jeans,
        underwear=items.gstring
    }
}

M.whistlergunnermale = scarlettables.copy(M.whistlergunnerfemale)
M.whistlergunnermale.name = "whistlergunnermale"
M.whistlergunnermale.attributes.gender = "male"
M.whistlergunnermale.printedname = "Whistler Gunner"
M.whistlergunnermale.combatname = "Whistler Gunner(M)"
M.whistlergunnermale.concisename = "WG(M)"
M.whistlergunnermale.clothing.underwear = items.underwear

M.whistlerthugfemale = characters.character {
    name="whistlerthugfemale",
    printedname="Whistler Thug",
    combatname="WhistlerThug(F)",
    concisename="WT(F)",
    attributes={gender="female"},
    choice=combat.basechoice(),
    statistics=statistics {
        strength=1,
        speed=1,
        power=0,
        maxenergy=1,
        energy=1
    },
    clothing=clothing {
        upper=items.tshirt,
        lower=items.skinnyjeans,
        underwear=items.boyshorts
    }
}

M.whistlerthugmale = scarlettables.copy(M.whistlerthugfemale)
M.whistlerthugmale.name = "whistlerthugmale"
M.whistlerthugmale.attributes.gender = "male"
M.whistlerthugmale.printedname = "Whistler Thug"
M.whistlerthugmale.combatname = "WhistlerThug(M)"
M.whistlerthugmale.concisename = "WT(M)"
M.whistlerthugmale.clothing.underwear = items.underwear

M.keiko = characters.character {
    name="keiko",
    printedname="Keiko",
    attributes={
        skinColor="caramel",
        height="short",
        musculature="soft",
        bodytype="slim",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down",
        gender="female"
    },
    clothing=clothing {
        upper=items.blouse,
        lower=items.skirt,
        underwear=items["lacy panties"]
    }
}

M.alexandra = characters.character {
    name="alexandra",
    printedname="Alexandra",
    attributes={
        gender="female",
        skinColor="tanned",
        height="tall",
        musculature="soft",
        bodytype="slender",
        eyeColor="blue",
        hairColor="brown",
        hairStyle="down"
    },
    clothing={
        upper=items["crop top"],
        lower=items["skirt"],
        underwear=items["lacy panties"]
    }
}

M.vanessa=characters.character {
    name="vanessa",
    printedname="Vanessa",
    combatname="Vanessa",
    description="A voluptuous, pale-skinned beauty.",
    attributes={
        gender="female",
        skinColor="ivory",
        height="average",
        musculature="soft",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="black",
        hairStyle="ponytail"
    },
    clothing={
        upper=items["t-shirt"],
        lower=items["daisy dukes"],
        underwear=items["lacy panties"]
    }
}

M.dana=characters.character {
    name="dana",
    printedname="Dana",
    description="A strange person in a fedora casing out Buzzsaw''s hideout.",
    attributes={
        gender="female",
        skinColor="peach",
        height="tall",
        musculature="fit",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down",
    },
    clothing={
        upper=items["blouse"],
        lower=items["slacks"],
        underwear=items["lacy panties"],
    }
}

M.delryn=characters.character {
    name="delryn",
    printedname="Delryn",
    description="A tall, broad-shouldered matronly person.",
    role="spanker",
    attackRanges="[]",
    attributes={
        gender="female",
        skinColor="ivory",
        height="tall",
        musculature="muscular",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing={
        upper=items["blouse"],
        lower=items["slacks"],
        underwear=items["underwear"]
    }
}

M.ironmaiden=characters.character {
    name="ironmaiden",
    printedname="Iron Maiden",
    description="A ex-Whistler of average height and build, with brown hair and a fit body.",
    attributes={
        gender="female",
        skinColor="tanned",
        height="average",
        musculature="fit",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing={
        upper=items["midriff-baring tanktop"],
        lower=items["cheeky daisy dukes"],
        underwear=items["thong"]
    }
}

M.marie=characters.character {
    name="marie",
    printedname="Marie",
    description="A strange person in a fedora casing Buzzsaw''s hideout.",
    attributes={
        gender="female",
        skinColor="tanned",
        height="average",
        musculature="fit",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing={
        upper=items["blouse"],
        lower=items["slacks"],
        underwear=items["thong"],
    }
}

M.natalie=characters.character {
    name="natalie",
    printedname="Natalie",
    description="A short, squat, big-bottomed nerd.",
    attributes={
        gender="female",
        skinColor="ivory",
        height="short",
        musculature="soft",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down",
    },
    clothing={
        upper=items["t-shirt"],
        lower=items["sweat pants"],
        underwear=items["underwear"],
    }
}

M.buzzsaw_ep1d2=characters.character {
    name="buzzsaw_ep1d2",
    printedname="Buzzsaw",
    combatname="Buzzsaw",
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.buzzsaw_ep1d2,
    description="A towering punk with a rainbow mohawk that can cut steel, the "
        .. "strength to lift cars, and an attitude to match.",
    attributes={
        gender="female",
        height="tall"
    },
    clothing=clothing {
        upper=items.studdedleatherjacket,
        lowere=items.blackjeanswithchain,
        underwear=items.studdedthong
    },
    powers={
        buzzsaw=buzzsawpowers.buzzsaw,
        chainwhip=buzzsawpowers.chainwhip,
        chainpull=buzzsawpowers.chainpull,
        soaringleap=buzzsawpowers.soaringleap,
        _keys={"buzzsaw", "chainwhip", "chainpull", "soaringleap"}
    },
    statistics={
        strength=4,
        speed=4,
        power=2,
        energy=50,
        maxenergy=50,
        _keys=statistickeys
    }
}

M["white dress"] = characters.character {
    name="white dress",
    printedname="White Dress",
    combatname="White Dress",
    pc=false,
    choice=ai.aichoice,
    description="A strange woman in a white dress.",
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["white dress"],
        lower=items["white dress"],
        underwear=items["thong"]
    },
    statistics=statistics {
        strength=1,
        speed=1,
        power=1,
        energy=1,
        maxenergy=1
    }
}

M["red dress"] = characters.character {
    name="red dress",
    printedname="Red Dress",
    combatname="Red Dress",
    pc=false,
    choice=ai.aichoice,
    description="A strange woman in a red dress.",
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["red dress"],
        lower=items["red dress"],
        underwear=items["thong"]
    },
    statistics=statistics {
        strength=1,
        speed=1,
        power=1,
        energy=1,
        maxenergy=1
    }
}

M["blue dress"] = characters.character {
    name="blue dress",
    printedname="Blue Dress",
    combatname="Blue Dress",
    pc=false,
    choice=ai.aichoice,
    description="A strange woman in a blue dress.",
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["blue dress"],
        lower=items["blue dress"],
        underwear=items["thong"]
    },
    statistics=statistics {
        strength=1,
        speed=1,
        power=1,
        energy=1,
        maxenergy=1
    }
}

local function spankdamage(attacker, target, combatstate)
    assert(false)
    local damage, retaliation, description = attack.defaultdamage(attacker, combatstate)
    if statusops.statusduration(t, statuses.otk) then
        local t = target
        local user = attacker
        description = user.combatname .. " tugs on " .. t.combatname
            .. "'s and swats " .. t.combatname .. "'s"
            .. "upthrust, helpless bottom with swift sharp smacks for "
            .. damage .. " damage!"
        if retalation > 0 then
            description = description .. t.combatname 
                .. " tries and fails to retaliate while over "
                .. attacker.combatname .. "'s knee!"
        end
        retalition = 0
        description = {description}
    end
    return damage, retaliation, description
end

M["succubus ep2d2"] = characters.character {
    name="succubus ep2d2",
    printedname="Succubus",
    combatname="Succubus",
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.succubus,
    description="A sultry dominatrix in a skimpy outfit with delusions of grandeur.",
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        power={count=1, page=power.power},
        _keys={"attack", "power"}
    },
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["succubus costume"],
        lower=items["succubus costume"],
        underwear=items["succubus costume"]
    },
    statistics=statistics {
        strength=7,
        speed=4,
        power=6,
        energy=85,
        maxenergy=85
    },
    powers={
        overmyknee=succubuspowers.overmyknee,
        cropwhip=succubuspowers.cropwhip,
        succuspank=succubuspowers.succuspank,
        _keys={
            "overmyknee",
            "cropwhip",
            "succuspank"
        }
    }
}

M["bat succubus minion"] = characters.character {
    name="bat succubus minion",
    printedname="Minion",
    combatname="Bat Minion(F)",
    description="One of Succubus' scantily clad minions, with a bat.",
    choice=ai.aichoice,
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    statistics=statistics {
        strength=9,
        speed=1,
        power=0,
        maxenergy=1,
        energy=1
    },
    clothing={
        upper=items["Succubus Minion Costume"],
        lower=items["Succubus Minion Costume"],
        underwear=items["Succubus Minion Costume"],
    }
}

M["gun succubus minion"] = characters.character {
    name="gun succubus minion",
    printedname="Minion",
    combatname="Gun Minion(F)",
    choice=ai.aichoice,
    choicecounts={
        shoot={count=1, page=scarletchoices.shoot},
        power={count=0, page=power.power},
        _keys={"shoot", "power"}
    },
    description="One of Succubus' scantily clad minions, with a stun gun.",
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down",
    },
    statistics=statistics {
        strength=12,
        speed=1,
        power=0,
        energy=1,
        maxenergy=1
    },
    clothing={
        upper=items["Succubus Minion Costume"],
        lower=items["Succubus Minion Costume"],
        underwear=items["Succubus Minion Costume"],
    }
}

M["unarmed succubus minion"]= characters.character {
    name="unarmed succubus minion",
    printedname="Minion",
    combatname="Minion(F)",
    description="One of Succubus' scantily clad minions, without a weapon.",
    choice=ai.aichoice,
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing={
        upper=items["Succubus Minion Costume"],
        lower=items["Succubus Minion Costume"],
        underwear=items["Succubus Minion Costume"]
    },
    statistics=statistics {
        strength=7,
        speed=1,
        power=0,
        energy=1,
        maxenergy=1
    }
}

M["succubus ep2d3"] = characters.character {
    name="succubus ep2d3",
    printedname="Succubus",
    combatname="Succubus",
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.succubus,
    description="A sultry dominatrix in a skimpy outfit with delusions of grandeur.",
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        power={count=1, page=power.power},
        _keys={"attack", "power"}
    },
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="fit",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["succubus costume"],
        lower=items["succubus costume"],
        underwear=items["succubus costume"]
    },
    statistics=statistics {
        strength=7,
        speed=4,
        power=7,
        energy=90,
        maxenergy=90
    },
    powers={
        overmyknee=succubuspowers.overmyknee,
        cropwhip=succubuspowers.cropwhip,
        succuspank=succubuspowers.succuspank,
        _keys={
            "overmyknee",
            "cropwhip",
            "succuspank"
        }
    }
}

M["bodyguard female"] = characters.character {
    name="bodyguard female",
    printedname="Bodyguard",
    combatname="Bodyguard(F)",
    concisename="B(F)",
    description="A broad shouldered individual in a dark blue uniform and "
        .. "wielding a stunner.",
    pc=false,
    choice=ai.aichoice,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="fit",
        bodytype="slim",
        eyeColor="brown",
        hairColor="brown",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["suit top"],
        lower=items["suit bottom"],
        underwear=items["cheeky boy shorts"]
    },
    statistics=statistics {
        strength=6,
        speed=8,
        power=0,
        energy=1,
        maxenergy=1
    }
}


M["bodyguard male"] = scarlettables.copy(M["bodyguard female"])
M["bodyguard male"].attributes.gender = "male"
M["bodyguard male"].name = "bodyguard male"
M["bodyguard male"].combatname = "Bodyguard(M)"
M["bodyguard male"].concisename = "B(M)"
M["bodyguard male"].clothing.underwear = items["underwear"]

M.poseidon_ep3d1 = characters.character {
    name="poseidon_ep3d1",
    printedname="Poseidon",
    combatname="Poseidon",
    concisename="Po",
    description="A young man in a speedo with water manipulation powers.",
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.poseidonep3d1,
    attributes={
        gender="male",
        skinColor="peach",
        height="tall",
        musculature="muscular",
        bodytype="slim",
        eyeColor="blue",
        hairColor="blue",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["chest"],
        lower=items["thong speedo"],
        underwear=items["thong speedo"]
    },
    statistics=statistics {
        strength=8,
        power=6,
        speed=3,
        energy=85,
        maxenergy=85
    },
    powers={
        hydrohammer=poseidonpowers.hydrohammer,
        hydroslide=poseidonpowers.hydroslide,
        chionfist=poseidonpowers.chionfist,
        chionskin=poseidonpowers.chionskin,
        _keys={
            "hydrohammer",
            "hydroslide",
            "chionfist",
            "chionskin"
        }
    }
}

M.prometheus_ep3d1 = characters.character {
    name="prometheus_ep3d1",
    printedname="Prometheus",
    combatname="Prometheus",
    concisename="Pr",
    description="A young woman in PVC with fire manipulation powers.",
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.prometheusep3d1,
    attributes={
        gender="female",
        skinColor="peach",
        height="tall",
        musculature="fit",
        bodytype="slim",
        eyeColor="red",
        hairColor="red",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["pvc bra"],
        lower=items["pvc hotpants"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=3,
        speed=8,
        power=6,
        energy=85,
        maxenergy=85
    },
    powers={
        twinserpents=prometheuspowers.twinserpents,
        fireball=prometheuspowers.fireball,
        _keys={
            "twinserpents",
            "fireball"
        }
    }
}

M.tempestas_ep3d1 = characters.character {
    name="tempestas_ep3d1",
    printedname="Tempestas",
    combatname="Tempestas",
    concisename="T",
    description="The Hurricane Heroine!",
    choice=combat.basechoice(),
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="green",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["tempestas tanktop"],
        lower=items["tempestas pants"],
        underwear=items["tempestas g-string"]
    },
    statistics = statistics {
        strength=3,
        speed=11,
        power=6,
        energy=100,
        maxenergy=100
    },
    powers={
        headwind=tempestaspowers.headwind,
        tailwind=tempestaspowers.tailwind,
        windtunnel=tempestaspowers.windtunnel,
        _keys={
            "headwind",
            "tailwind",
            "windtunnel"
        }
    }
}

M.buzzsaw_ep3d1 = characters.character {
    name="buzzsaw_ep3d1",
    printedname="Buzzsaw",
    combatname="Buzzsaw",
    statistics={
        strength=13,
        speed=9,
        power=4,
        maxenergy=130,
        energy=130,
        _keys=statistickeys
    },
    powers={
        buzzsaw=buzzsawpowers.buzzsaw,
        chainwhip=buzzsawpowers.chainwhip,
        _keys={"buzzsaw", "chainwhip", }
    },
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.buzzsaw,
    attributes={
        gender="female",
        height="tall",
    },
    clothing=clothing {
        upper=items.studdedleatherjacket,
        lowere=items.blackjeanswithchain,
        underwear=items.studdedthong
    }
}

M.poseidon_ep3d2 = characters.character {
    name="poseidon_ep3d2",
    printedname="Poseidon",
    combatname="Poseidon",
    concisename="Po",
    description="A young man in a speedo with water manipulation powers.",
    pc=false,
    choice=ai.aichoice,
    ai=scarletai.poseidonep3d2,
    attributes={
        gender="male",
        skinColor="peach",
        height="tall",
        musculature="muscular",
        bodytype="slim",
        eyeColor="blue",
        hairColor="blue",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["chest"],
        lower=items["thong speedo"],
        underwear=items["thong speedo"]
    },
    statistics=statistics {
        strength=10,
        power=8,
        speed=6,
        energy=120,
        maxenergy=120
    },
    powers={
        hydrohammer=poseidonpowers.hydrohammer,
        hydroslide=poseidonpowers.hydroslide,
        chionfist=poseidonpowers.chionfist,
        chionskin=poseidonpowers.chionskin,
        hydrogator=poseidonpowers.hydrogator,
        _keys={
            "hydrohammer",
            "hydroslide",
            "chionfist",
            "chionskin",
            "hydrogator"
        }
    }
}

M.prometheus_ep3d2 = characters.character {
    name="prometheus_ep3d2",
    printedname="Prometheus",
    combatname="Prometheus",
    concisename="Pr",
    description="A young woman in PVC with fire manipulation powers.",
    pc=true,
    choice=ai.aichoice,
    ai=scarletai.prometheusep3d2,
    attributes={
        gender="female",
        skinColor="peach",
        height="tall",
        musculature="fit",
        bodytype="slim",
        eyeColor="red",
        hairColor="red",
        hairStyle="down"
    },
    clothing=clothing {
        upper=items["pvc bra"],
        lower=items["pvc hotpants"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=3,
        speed=11,
        power=10,
        energy=120,
        maxenergy=120
    },
    powers={
        twinserpents=prometheuspowers.twinserpents,
        fireball=prometheuspowers.fireball,
        flamey=prometheuspowers.flamey,
        _keys={
            "twinserpents",
            "fireball",
            "flamey"
        }
    }
}

M.tempestas_ep3d2 = characters.character {
    name="tempestas_ep3d2",
    printedname="Tempestas",
    combatname="Tempestas",
    concisename="T",
    description="The Hurricane Heroine!",
    choice=combat.basechoice(),
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="green",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["tempestas tanktop"],
        lower=items["tempestas pants"],
        underwear=items["tempestas g-string"]
    },
    statistics = statistics {
        strength=3,
        speed=12,
        power=9,
        energy=120,
        maxenergy=120
    },
    powers={
        headwind=tempestaspowers.headwind,
        tailwind=tempestaspowers.tailwind,
        windtunnel=tempestaspowers.windtunnel,
        _keys={
            "headwind",
            "tailwind",
            "windtunnel"
        }
    }
}

M.tempestas_ep3d2_1 = characters.character {
    name="tempestas_ep3d2_1",
    printedname="Tempestas",
    combatname="Tempestas",
    concisename="T",
    description="The Hurricane Heroine!",
    choice=combat.basechoice(),
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="green",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["tempestas tanktop"],
        lower=items["tempestas pants"],
        underwear=items["tempestas g-string"]
    },
    statistics = statistics {
        strength=3,
        speed=12,
        power=9,
        energy=120,
        maxenergy=120
    },
    powers={
        headwind=tempestaspowers.headwind,
        tailwind=tempestaspowers.tailwind,
        windtunnel=tempestaspowers.windtunnel,
        yomama=tempestaspowers.yomama,
        _keys={
            "headwind",
            "tailwind",
            "windtunnel",
            "yomama"
        }
    }
}

M.princess = characters.character {
    name="princess",
    printedname="Princess",
    combatname="Princess",
    concisename="P",
    description="A childlike brat who can control toys.",
    choice=ai.aichoice,
    ai=princessai.princess,
    choicecounts={
        power={count=2, page=power.power},
        _keys={"power"}
    },
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="muscular",
        bodytype="slim",
        eyeColor="blue",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["pretty prink dress"],
        lower=items["pretty prink dress"],
        underwear=items["bear-dotted panties"]
    },
    statistics = statistics {
        strength=0,
        speed=0,
        power=27,
        energy=135,
        maxenergy=135
    },
    powers={
        summontoys=princesspowers.princess.summontoys,
        strengthentoys=princesspowers.princess.strengthentoys,
        bearswipe=princesspowers.princess.bearswipe,
        bearshield=princesspowers.princess.bearshield,
        healbear=assert(princesspowers.princess.healbear),
        _keys={
            "summontoys",
            "strengthentoys",
            "bearswipe",
            "bearshield",
            "healbear"
        }
    }
}

M.mrcuddles = characters.character {
    name="mrcuddles",
    printedname="Mr. Cuddles",
    combatname="Mr. Cuddles",
    concisename="C",
    description="A giant clay bear with human hands created by Princess.",
    choice=ai.aichoice,
    ai=princessai.mrcuddles,
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        power={count=1, page=power.power},
        _keys={"attack", "power"}
    },
    type="toy",
    huge=true,
    pc=true,
    attributes={
        gender="male",
        skinColor="peach",
        height="gargantuan",
        musculature="muscular",
        bodytype="heavyset",
        eyeColor="black",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["bare chest"],
        lower=items["bottom"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=14,
        speed=8,
        power=0,
        energy=110,
        maxenergy=110
    },
    powers={
        bearcounter=princesspowers.mrcuddles.bearcounter,
        bearpunch=princesspowers.mrcuddles.bearpunch,
        beargrab=princesspowers.mrcuddles.beargrab,
        bearspank=princesspowers.mrcuddles.bearspank,
        _keys={
            "bearcounter",
            "bearpunch",
            "beargrab",
            "bearspank"
        }
    }
}

M.mrcuddles_superpowered = scarlettables.copy(M.mrcuddles)
M.mrcuddles_superpowered.name = "mrcuddles_superpowered"
M.mrcuddles_superpowered.statistics = statistics {
    strength=19+5,
    speed=8,
    power=0,
    energy=80,
    maxenergy=185
}


M.stuffed_bear = characters.character {
    name="stuffed_bear",
    printedname="Sir Stuffins",
    combatname="Sir Stuffins",
    concisename="SS",
    description="An animated stuffed bear wielding a pool noodle as a lance.",
    choice=ai.aichoice,
    ai=princessai.sirstuffins,
    pc=true,
    type="toy",
    attributes={
        gender="male",
        skinColor="peach",
        height="tall",
        musculature="muscular",
        bodytype="heavyset",
        eyeColor="black",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["bare chest"],
        lower=items["bottom"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=15,
        speed=8,
        power=0,
        energy=115,
        maxenergy=115
    },
    powers={
        charge=princesspowers.stuffins.charge,
        ["noodle_whack"]=princesspowers.stuffins.noodle_whack,
        ["noodle_spank"]=princesspowers.stuffins.noodle_spank,
        _keys={
            "charge",
            "noodle_whack",
            "noodle_spank"
        }
    }
}

M.stuffed_bear_ep4d2 = scarlettables.copy(M.stuffed_bear)
M.stuffed_bear_ep4d2.name = "stuffed_bear_ep4d2"
M.stuffed_bear_ep4d2.statistics = statistics {
    strength=math.floor(M.stuffed_bear.statistics.strength / 2),
    speed=math.floor(M.stuffed_bear.statistics.speed / 2),
    power=math.floor(M.stuffed_bear.statistics.power / 2),
    energy=math.floor(M.stuffed_bear.statistics.energy / 2),
    maxenergy=math.floor(M.stuffed_bear.statistics.maxenergy / 2),
}

M.stuffed_giraffe = characters.character {
    name="stuffed_giraffe",
    printedname="Giraffalot",
    combatname="Giraffalot",
    concisename="G",
    description="A stuffed giraffe serving as Sir Stuffin's steed.",
    choice=ai.aichoice,
    ai=princessai.giraffalot,
    pc=true,
    type="toy",
    attributes={
        gender="female",
        skinColor="peach",
        height="gargantuan",
        musculature="slender",
        bodytype="slim",
        eyeColor="black",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["bare chest"],
        lower=items["bottom"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=8,
        speed=15,
        power=0,
        energy=115,
        maxenergy=115
    },
    powers={
        rear=princesspowers.giraffalot.rear,
        trample=princesspowers.giraffalot.trample,
        pin=princesspowers.giraffalot.pin,
        _keys={
            "rear",
            "trample",
            "pin",
        }
    }
}


M.stuffed_giraffe_ep4d2 = scarlettables.copy(M.stuffed_giraffe)
M.stuffed_giraffe_ep4d2.name = "stuffed_giraffe_ep4d2"
M.stuffed_giraffe_ep4d2.statistics = statistics {
    strength=math.floor(M.stuffed_giraffe.statistics.strength / 2),
    speed=math.floor(M.stuffed_giraffe.statistics.speed / 2),
    power=math.floor(M.stuffed_giraffe.statistics.power / 2),
    energy=math.floor(M.stuffed_giraffe.statistics.energy / 2),
    maxenergy=math.floor(M.stuffed_giraffe.statistics.maxenergy / 2),
}


M.legostopheles = characters.character {
    name="legostopheles",
    printedname="Legostopheles",
    combatname="Legostopheles",
    concisename="L",
    description="A giant tentacled, multi-armed demon made of legos.",
    choice=ai.aichoice,
    ai=princessai.legostopheles,
    type="toy",
    choicecounts={
        attack={count=6, page=scarletchoices.attack},
        power={count=1, page=power.power},
        _keys={"attack", "power"}
    },
    pc=true,
    attributes={
        gender="male",
        skinColor="peach",
        height="gargantuan",
        musculature="muscular",
        bodytype="heavyset",
        eyeColor="green",
        hairColor="red"
    },
    clothing=clothing {
        upper=items["bare chest"],
        lower=items["bottom"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=20,
        speed=6,
        power=0,
        energy=130,
        maxenergy=130
    },
    powers={
        grab=princesspowers.legostopheles.grab,
        legoswirl=princesspowers.legostopheles.legoswirl,
        legopaddle=princesspowers.legostopheles.legopaddle,
        _keys={
            "grab",
            "legoswirl",
            "legopaddle"
        }
    }
}


M.legostopheles_ep4d2 = scarlettables.copy(M.legostopheles)
M.legostopheles_ep4d2.name = "legostopheles_ep4d2"
M.legostopheles_ep4d2.statistics = statistics {
    strength=math.floor(M.legostopheles.statistics.strength / 2),
    speed=math.floor(M.legostopheles.statistics.speed / 2),
    power=math.floor(M.legostopheles.statistics.power / 2),
    energy=math.floor(M.legostopheles.statistics.energy / 2),
    maxenergy=math.floor(M.legostopheles.statistics.maxenergy / 2),
}
M.legostopheles_ep4d2.pc = true


M.legostopheles_superpowered = scarlettables.copy(M.legostopheles)
M.legostopheles_superpowered.name = "legostopheles_superpowered"
M.legostopheles_superpowered.statistics = statistics {
    strength=20+5,
    speed=6,
    power=0,
    energy=30,
    maxenergy=180
}

M.drone = characters.character {
    name="drone",
    printedname="Drone",
    combatname="Drone",
    concisename="D",
    description="A small drone possessed by a malevolent power.",
    choice=ai.aichoice,
    type="toy",
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        _keys={"attack"}
    },
    pc=false,
    attributes={
        gender="male",
        skinColor="peach",
        height="gargantuan",
        musculature="muscular",
        bodytype="heavyset",
        eyeColor="green",
        hairColor="red"
    },
    clothing=clothing {
        upper=items["bare chest"],
        lower=items["bottom"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=5,
        speed=10,
        power=0,
        energy=1,
        maxenergy=1
    }
}

M.actionfigure = characters.character {
    name="actionfigure",
    printedname="Action Figure",
    combatname="Action Figure",
    concisename="AF",
    description="A small plastic action figure possessed by a malevolent power.",
    choice=ai.aichoice,
    type="toy",
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        _keys={"attack"}
    },
    pc=false,
    attributes={
        gender="male",
        skinColor="peach",
        height="short",
        musculature="muscular",
        bodytype="heavyset",
        eyeColor="brown",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["bare chest"],
        lower=items["bottom"],
        underwear=items["bottom"]
    },
    statistics = statistics {
        strength=10,
        speed=5,
        power=0,
        energy=1,
        maxenergy=1
    }
}

M.tempestas_ep4d1 = characters.character {
    name="tempestas_ep4d1",
    printedname="Tempestas",
    combatname="Tempestas",
    concisename="T",
    description="The Hurricane Heroine!",
    choice=combat.basechoice(),
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="green",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["tempestas tanktop"],
        lower=items["tempestas pants"],
        underwear=items["tempestas g-string"]
    },
    statistics = statistics {
        strength=4,
        speed=13,
        power=10,
        energy=135,
        maxenergy=135
    },
    powers={
        headwind=tempestaspowers.headwind,
        tailwind=tempestaspowers.tailwind,
        windtunnel=tempestaspowers.windtunnel,
        yomama=tempestaspowers.yomama,
        twister=tempestaspowers.twister,
        airburst=tempestaspowers.airburst,
        _keys={
            "headwind",
            "tailwind",
            "yomama",
            "windtunnel",
            "twister",
            "airburst"
        }
    }
}

M.tempestas_ep4d1 = characters.character {
    name="tempestas_ep4d1",
    printedname="Tempestas",
    combatname="Tempestas",
    concisename="T",
    description="The Hurricane Heroine!",
    choice=combat.basechoice(),
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="green",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["tempestas tanktop"],
        lower=items["tempestas pants"],
        underwear=items["tempestas g-string"]
    },
    statistics = statistics {
        strength=4,
        speed=13,
        power=10,
        energy=135,
        maxenergy=135
    },
    powers={
        headwind=tempestaspowers.headwind,
        tailwind=tempestaspowers.tailwind,
        windtunnel=tempestaspowers.windtunnel,
        yomama=tempestaspowers.yomama,
        twister=tempestaspowers.twister,
        airburst=tempestaspowers.airburst,
        _keys={
            "headwind",
            "tailwind",
            "yomama",
            "windtunnel",
            "twister",
            "airburst"
        }
    }
}

M.tempestas_ep4d2 = characters.character {
    name="tempestas_ep4d2",
    printedname="Tempestas",
    combatname="Tempestas",
    concisename="T",
    description="The Hurricane Heroine!",
    choice=combat.basechoice(),
    pc=true,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="green",
        hairColor="blonde"
    },
    clothing=clothing {
        upper=items["tempestas tanktop"],
        lower=items["tempestas pants"],
        underwear=items["tempestas g-string"]
    },
    statistics = statistics {
        strength=4,
        speed=14,
        power=11,
        energy=145,
        maxenergy=145
    },
    powers={
        headwind=tempestaspowers.headwind,
        tailwind=tempestaspowers.tailwind,
        windtunnel=tempestaspowers.windtunnel,
        yomama=tempestaspowers.yomama,
        twister=tempestaspowers.twister,
        airburst=tempestaspowers.airburst,
        _keys={
            "headwind",
            "tailwind",
            "yomama",
            "windtunnel",
            "twister",
            "airburst"
        }
    }
}

M.anklyana_ep4d2 = characters.character {
    name="anklyana_ep4d2",
    printedname="Anklyana",
    combatname="Anklyana",
    concisename="A",
    description="An impenetrable wall of a heroine",
    choice=combat.basechoice(),
    pc=true,
    choicecounts={
        attack={count=0, page=scarletchoices.attack},
        power={count=2, page=power.power},
        _keys={"attack", "power"}
    },
    attributes={
        gender="female",
        skinColor="peach",
        height="short",
        musculature="soft",
        bodytype="voluptuous",
        eyeColor="brown",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["anklyana leotard"],
        lower=items["anklyana leotard"],
        underwear=items["anklyana leotard"]
    },
    statistics = statistics {
        strength=1,
        speed=1,
        power=27,
        energy=174,
        maxenergy=174
    },
    powers={
        armorsaurus=anklyanapowers.armorsaurus,
        bubblesaurus=anklyanapowers.bubblesaurus,
        wallsaurus=anklyanapowers.wallsaurus,
        _keys={
            "armorsaurus",
            "bubblesaurus",
            "wallsaurus"
        }
    }
}

M.anklyana_ep5d1 = scarlettables.copy(M.anklyana_ep4d2)
M.anklyana_ep5d1.name = "anklyana_ep5d1"
M.anklyana_ep5d1.statistics = statistics {
    strength=1,
    speed=1,
    power=30,
    energy=192,
    maxenergy=192
}

M.smuggler = characters.character {
    name="smuggler",
    printedname="Smuggler",
    combatname="Smuggler",
    concisename="S",
    description="A generic smuggler with a particle gun",
    choice=combat.basechoice(),
    choicecounts={
        shoot={count=1, page=scarletchoices.shoot},
        _keys={"shoot"}
    },
    attributes={
        gender="male",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["t-shirt"],
        lower=items["jeans"],
        underwear=items["boxers"]
    },
    statistics = statistics {
        strength=1,
        speed=1,
        power=5,
        energy=1,
        maxenergy=1
    }
}

M.smuggler1 = scarlettables.copy(M.smuggler)
M.smuggler1.name = "smuggler1"


M.big_guns = characters.character {
    name="big_guns",
    printedname="Big Guns",
    combatname="Big Guns",
    concisename="B",
    description="A smuggler who appears to possess some superhuman strength.",
    choice=combat.basechoice(),
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        _keys={"attack"}
    },
    attributes={
        gender="male",
        skinColor="peach",
        height="average",
        musculature="muscular",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["t-shirt"],
        lower=items["jeans"],
        underwear=items["boxers"]
    },
    statistics = statistics {
        strength=15,
        speed=1,
        power=1,
        energy=75,
        maxenergy=75
    }
}


local names = {}
for _, character in pairs(M) do
    assert(
        not names[character.name],
        "Two characters have the name: " .. character.name
    )
    names[character.name] = true
    if character.statistics then
        assert(
            character.statistics._keys,
            character.name .. " is missing their statistic keys."
        )
    end
end


M.anklyana_ep5d2 = scarlettables.copy(M.anklyana_ep5d1)
M.anklyana_ep5d2.name = "anklyana_ep5d2"
M.anklyana_ep5d2.statistics = statistics {
    strength=1,
    speed=1,
    power=33,
    energy=210,
    maxenergy=210
}
M.anklyana_ep5d2.powers.stasisaurus = anklyanapowers.stasisaurus
M.anklyana_ep5d2.powers.empathysaurus = anklyanapowers.empathysaurus
table.insert(M.anklyana_ep5d2.powers._keys, "stasisaurus")
table.insert(M.anklyana_ep5d2.powers._keys, "empathysaurus")

M.tempestas_ep5d2 = scarlettables.copy(M.tempestas_ep4d2)
M.tempestas_ep5d2.name = "tempestas_ep4d2"
M.tempestas_ep5d2.statistics = statistics {
    strength=5,
    speed=17,
    power=14,
    energy=180,
    maxenergy=180
}

M.leopard_ep5d2 = characters.character {
    name="leopard_ep5d2",
    printedname="Leopard",
    combatname="Leopard",
    pc=false,
    description="A superpowered huntress.",
    choice=ai.aichoice,
    ai=scarletai.leopardep5d2,
    attributes={
        gender="female",
        skinColor="peach",
        height="average",
        musculature="fit",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["leopard-print bra"],
        lower=items["leopard-print leggings"],
        underwear=items.thong
    },
    statistics = statistics {
        strength=20,
        speed=20,
        power=20,
        energy=300,
        maxenergy=300
    },
    powers={
        distortion=leopardpowers.distortion,
        roundhousekick=leopardpowers.roundhousekick,
        throw=leopardpowers.throw,
        judo=leopardpowers.judo,
        _keys={"distortion", "roundhousekick", "throw", "judo"}
    }
}

M.bright_ep6d1 = characters.character {
    name="bright_ep6d1",
    printedname="Bright",
    combatname="Bright",
    pc=false,
    description="A young woman with light themed, angelic powers",
    choice=ai.aichoice,
    ai=scarletai.brightep6d1,
    attributes={
        gender="female",
        skinColor="peach",
        height="tall",
        musculature="fit",
        bodytype="slender",
        eyeColor="blue",
        hairColor="brown"
    },
    clothing=clothing {
        upper=items["bright bustier"],
        lower=items["bright skirt"],
        underwear=items["bright thong"]
    },
    choicecounts={
        shoot={count=1, page=scarletchoices.shoot},
        power={count=1, page=power.power},
        _keys={"shoot", "power"}
    },
    statistics = statistics {
        strength=10,
        speed=19,
        power=10,
        energy=117,
        maxenergy=117
    },
    powers={
        brightsaber=brightpowers.brightsaber,
        brightwings=brightpowers.brightwings,
        flashingcleave=brightpowers.flashingcleave,
        blindingcharge=brightpowers.blindingcharge,
        _keys={"brightsaber", "brightwings", "flashingcleave", "blindingcharge"}
    }
}

M.supersuit_police = {
    name="supersuit_police",
    printedname="Supersuited Officer",
    combatname="Supersuited Officer",
    pc=false,
    description="A police officer in enhanced gear.",
    choice=combat.basechoice(),
    attributes={
        gender="male",
        skinColor="peach",
        height="tall",
        musculature="muscular",
        bodytype="slender",
        eyeColor="brown",
        hairColor="brown"
    },
    choicecounts={
        attack={count=1, page=scarletchoices.attack},
        shoot={count=1, page=scarletchoices.shoot},
        power={count=1, page=power.power},
        _keys={"shoot", "power"}
    },
    statistics = statistics {
        strength=20,
        speed=5,
        power=1,
        energy=10,
        maxenergy=10
    },
}

return M
